/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.modelling;

import java.io.IOException;

import de.jtem.java2d.Annotation;
import de.jtem.java2d.SceneComponent;
import de.jtem.modelling.ActionTree;

public class DraggableAnnotation extends ModellingTool2D {
	private static final long serialVersionUID = 1L;
	
	private Annotation annotation;
	private double storedX;
	private double storedY;
	private int storedAnchor;
	private String storedText;

	public DraggableAnnotation() {
		this(new Annotation());
	}
	
	/** makes a copy of the model of f */
	
	public DraggableAnnotation(DraggableAnnotation f) {
		super(f);
		annotation = new Annotation(f.getModel());
		getViewScene().copyAttributesFrom(f.getViewScene());
		setUp();
	}

	/** a is not copied */
	
	public DraggableAnnotation(Annotation a) {
		annotation = a;
		setUp();
	}
	
	private void setUp() {
		// for serialization
		getViewScene().getAnnotations().clear();
		getViewScene().getAnnotations().add(annotation);
		getViewScene().setAnnotationDragEnabled(true);
		getViewScene().addDragListener(new TranslationTool(this));
		storeModelState();
	}
	
	public Annotation getModel() {
		return annotation;
	}

	public SceneComponent getControlScene() {
		return null;
	}

	public void translate(double x, double y) {
		annotation.setLocation(x + annotation.getX(), y + annotation.getY());
		updateFromModel();
	}

	public void reset() {}

	public void storeModelState() {
		storedX = annotation.getX();
		storedY = annotation.getY();
		storedAnchor = annotation.getAnchor();
		storedText = annotation.getText();
	}

	public void recallModelState() {
		annotation.setText(storedText);
		annotation.setAnchor(storedAnchor);
		annotation.setLocation(storedX, storedY);
	}

	public ActionTree getActionTree() {
		return null;
	}
	
	public DraggableAnnotation clone() {
		return new DraggableAnnotation(this);
	}
	
	protected void updateFromModel() {}
	
	private void readObject(java.io.ObjectInputStream stream)
	throws IOException, ClassNotFoundException {
		stream.defaultReadObject();
		setUp();
	}
}
