/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.modelling;

import java.io.IOException;

import de.jtem.java2d.Annotation;
import de.jtem.java2d.DragListener;
import de.jtem.java2d.Icon2D;
import de.jtem.java2d.SceneComponent;
import de.jtem.java2d.TransformedMouseEvent;
import de.jtem.modelling.ActionTree;

public class DraggableLatexIcon extends ModellingTool2D {
	private static final long serialVersionUID = 1L;
	
	private LatexIcon latexIcon;
	private Icon2D icon2D;
	private double storedX;
	private double storedY;
	private int storedAnchor;
	private String storedLatexSource;

	public DraggableLatexIcon() {
		this(new LatexIcon(),0,0,Annotation.CENTER);
	}
	
	/** makes a copy of the model of f */
	
	public DraggableLatexIcon(DraggableLatexIcon f) {
		super(f);
		latexIcon = new LatexIcon(f.getModel());
		icon2D = new Icon2D(latexIcon.getIcon(), f.getX(), f.getY(), f.getAnchor());
		getViewScene().copyAttributesFrom(f.getViewScene());
		setUp();
	}

	/** p is not copied */
	
	public DraggableLatexIcon(LatexIcon i, double x, double y, int anchor) {
		latexIcon = i;
		icon2D = new Icon2D(latexIcon.getIcon(), x, y, anchor);
		setUp();
	}
	
	private void setUp() {
		getViewScene().getIcons().clear();
		getViewScene().getIcons().add(icon2D);
		getViewScene().setIconDragEnabled(true);
		getViewScene().addDragListener(new DragListener() {
			double lastX;
			double lastY;
			public void dragStart(TransformedMouseEvent e) {
				lastX = e.getX();
				lastY = e.getY();
			}
			public void drag(TransformedMouseEvent e) {
				double newX = e.getX();
				double newY = e.getY();
				
				icon2D.setLocation(
						icon2D.getX() + newX - lastX,
						icon2D.getY() + newY - lastY
				);
				lastX = newX;
				lastY = newY;
				fireModelChange();
			}
			public void dragEnd(TransformedMouseEvent e) {}
		});
		storeModelState();
	}
	
	public LatexIcon getModel() {
		return latexIcon;
	}

	public SceneComponent getControlScene() {
		return null;
	}

	public void translate(double x, double y) {
		icon2D.setLocation(x + icon2D.getX(), y + icon2D.getY());
		updateFromModel();
	}

	public void reset() {}

	public void storeModelState() {
		storedX = icon2D.getX();
		storedY = icon2D.getY();
		storedAnchor = icon2D.getAnchor();
		storedLatexSource = latexIcon.getLatexSource();
	}

	public void recallModelState() {
		latexIcon.setLatexSource(storedLatexSource);
		icon2D.setAnchor(storedAnchor);
		icon2D.setLocation(storedX, storedY);
	}
	
	public void updateFromModel() {
		icon2D.setIcon(latexIcon.getIcon());
	}

	public ActionTree getActionTree() {
		return null;
	}
	
	public DraggableLatexIcon clone() {
		return new DraggableLatexIcon(this);
	}

	public double getX() {
		return icon2D.getX();
	}

	public void setX(double x) {
		icon2D.setX(x);
	}

	public double getY() {
		return icon2D.getY();
	}

	public void setY(double y) {
		icon2D.setY(y);
	}
	
	public int getAnchor() {
		return icon2D.getAnchor();
	}

	public void setAnchor(int a) {
		icon2D.setAnchor(a);
	}
	
	private void readObject(java.io.ObjectInputStream stream)
	throws IOException, ClassNotFoundException {
		stream.defaultReadObject();
		setUp();
	}
}
