/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.modelling;


import java.io.IOException;

import de.jtem.java2d.Annotation;
import de.jtem.java2d.SceneComponent;
import de.jtem.java2dx.Point2DDouble;

public class DraggablePoint2D extends ModellingTool2D {
	private static final long serialVersionUID = -5623989232010175729L;
	private Point2DDouble point;
	private Annotation annotation = new Annotation();
	private Point2DDouble storedPoint = new Point2DDouble();

	public DraggablePoint2D() {
		this(new Point2DDouble());
	}
	
	public DraggablePoint2D(DraggablePoint2D p, Point2DDouble q) {
		super(p);
		q.setLocation(p.getModel());
		point = q;
		getViewScene().copyAttributesFrom(p.getViewScene());
		setUp();
	}
	
	/** makes a copy of the model of p */
	
	public DraggablePoint2D(DraggablePoint2D p) {
		this(p,new Point2DDouble(p.getModel()));
	}

	/** p is not copied */
	
	public DraggablePoint2D(Point2DDouble p) {
		point = p;
		setUp();
	}
	
	private void setUp() {
		getViewScene().getPoints().add(point);
		getViewScene().getAnnotations().add(annotation);
		getViewScene().addDragListener(new TranslationTool(this));
		storeModelState();
		updateFromModel();
	}
	
	public Point2DDouble getModel() {
		return point;
	}

	public SceneComponent getControlScene() {
		return null;
	}

	public void translate(double x, double y) {
		point.setLocation(point.x + x, point.y + y);
		updateFromModel();
	}

	public void storeModelState() {
		storedPoint.setLocation(getModel());
	}

	public void recallModelState() {
		getModel().setLocation(storedPoint);
		updateFromModel();
	}
	
	public void updateFromModel() {
		annotation.setLocation(point);
	}
	
	public DraggablePoint2D clone() {
		return new DraggablePoint2D(this);
	}
	
	private void readObject(java.io.ObjectInputStream stream)
	throws IOException, ClassNotFoundException {
		stream.defaultReadObject();
		setUp();
	}

	public int getAnchor() {
		return annotation.getAnchor();
	}

	public void setAnchor(int a) {
		annotation.setAnchor(a);
	}

	public String getText() {
		return annotation.getText();
	}

	public void setText(String s) {
		annotation.setText(s);
	}
}
