/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.modelling;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.jtem.java2d.SceneComponent;
import de.jtem.modelling.Modeller;

public class GraphicsModeller2DPanel extends JPanel {
	private static final long serialVersionUID = -8763475444576248223L;
	private GraphicsModeller2D modeller;
	private SpinnerNumberModel snapModel;
	private JCheckBox snapCheckBox;
	private JCheckBox gridCheckBox;
	private JCheckBox controlsCheckBox;
	
	public GraphicsModeller2DPanel() {
		setLayout(new BorderLayout());
		modeller = new GraphicsModeller2D();
		add(modeller.getViewer());
		
		JPanel separator = new javax.swing.JPanel();
		BevelBorder b = new BevelBorder(BevelBorder.LOWERED);
		separator.setMinimumSize(
				new java.awt.Dimension(0,5)
		);
		separator.setPreferredSize(
				new java.awt.Dimension(32767,5)
		);
		separator.setMaximumSize(
				new java.awt.Dimension(32767,5)
		);
		separator.setBorder(b);
		Box controlPanel = new Box(BoxLayout.Y_AXIS);
		controlPanel.add(separator);
		Box box = new Box(BoxLayout.X_AXIS);
		Box snapBox = new Box(BoxLayout.X_AXIS);
		JLabel snapLabel = new  JLabel("snap resolution:");
		snapLabel.setBorder(new EmptyBorder(0,5,0,5));
		snapModel = new SpinnerNumberModel(
				new Integer(modeller.getViewer().getSnapToGridResolution()),
				null,
				null,
				new Integer(1)
		);
		
		snapModel.addChangeListener(new ChangeListener() {
			
			public void stateChanged(ChangeEvent e) {
				modeller.getViewer().setSnapToGridResolution(snapModel.getNumber().intValue());
			}
		});
		JSpinner snap = new JSpinner(snapModel);
		snap.setPreferredSize(new Dimension(60,20));
		snap.setMaximumSize(new Dimension(60,20));
		snapBox.add(snapLabel);
		snapBox.add(snap);
		box.add(snapBox);
		snapCheckBox = new JCheckBox("snap to grid");
		snapCheckBox.setSelected(modeller.getViewer().isSnapToGridEnabled());
		snapCheckBox.setBorder(new EmptyBorder(0,8,0,5));
		snapCheckBox.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				System.out.println("snap listener "+snapCheckBox.isSelected());
				modeller.getViewer().setSnapToGridEnabled(snapCheckBox.isSelected());
			}
		});
		box.add(snapCheckBox);
		box.add(Box.createHorizontalGlue());
		controlsCheckBox = new JCheckBox("show controls");
		SceneComponent c = (SceneComponent) modeller.getModeller().getControlScene();
		controlsCheckBox.setSelected(c.isVisible());
		controlsCheckBox.setBorder(new EmptyBorder(0,8,0,5));
		controlsCheckBox.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				SceneComponent c = (SceneComponent) modeller.getModeller().getControlScene();
				System.out.println("setting visible "+controlsCheckBox.isSelected());
				c.setVisible(controlsCheckBox.isSelected());
				modeller.getViewer().repaint();
			}
		});
		box.add(controlsCheckBox);
		gridCheckBox = new JCheckBox("show coordinate grid");
		gridCheckBox.setSelected(modeller.getViewer().isGridEnabled());
		gridCheckBox.setBorder(new EmptyBorder(0,8,0,5));
		gridCheckBox.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				modeller.getViewer().setGridEnabled(gridCheckBox.isSelected());
			}
		});
		box.add(gridCheckBox);
		box.setPreferredSize(new Dimension(32767,25));
		controlPanel.add(box);
		add(controlPanel,BorderLayout.SOUTH);
	}
	
	public GraphicsModeller2D getGraphicsModeller2D() {
		return modeller;
	}
	
	public static void main(String[] args) {
		final GraphicsModeller2DPanel graphicsModellerPanel =
			new GraphicsModeller2DPanel();
		JFrame frame = new JFrame("graphics modeller 2D");
		JMenuBar bar = new JMenuBar();
		frame.setJMenuBar(bar);
		Modeller modeller = graphicsModellerPanel.getGraphicsModeller2D().getModeller();
		bar.add(modeller.getFileMenu().asMenuItem());
		bar.add(modeller.getEditMenu().asMenuItem());
		bar.add(modeller.getInspectMenu().asMenuItem());
		
		frame.getContentPane().setLayout(new BorderLayout());
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(600,600);
		
		frame.getContentPane().add(graphicsModellerPanel);
		frame.setVisible(true);
	}
}
