/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.modelling;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.beans.Customizer;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;



public class LatexIconCustomizer extends JPanel implements Customizer {
	
	private static final long serialVersionUID = 4858799775059664599L;
	private PropertyChangeSupport support = new PropertyChangeSupport(this);
	private JTextArea log = new JTextArea();
	private JTextArea input = new JTextArea();
	private JLabel preview = new JLabel();
	private LatexIcon latexIcon;
	private SpinnerNumberModel fontSizeModel;
	JFrame logFrame;
	
	public LatexIconCustomizer() {this(null);}
	
	public LatexIconCustomizer(LatexIcon f) {
		super(new BorderLayout());
		
		// log frame
		logFrame = new JFrame("log");
		JScrollPane logScroll = new JScrollPane(log);
		logFrame.setSize(400,400);
		logFrame.setLocationRelativeTo(null);
		logFrame.add(logScroll);
		
		Box controls = new Box(BoxLayout.X_AXIS);
		Box fontSizeBox = new Box(BoxLayout.X_AXIS);
		JLabel fontSizeLabel = new JLabel("font size:");
		fontSizeLabel.setBorder(new EmptyBorder(0,5,0,5));
		fontSizeBox.add(fontSizeLabel);
		fontSizeModel = new SpinnerNumberModel(
					new Integer(f == null ? 22 : f.getFontSize()),
					new Integer(0),
					null,
					new Integer(1)
				);
		fontSizeModel.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				fontSizeChanged();
			}
		});
		JSpinner dpiSpinner = new JSpinner(fontSizeModel);
		dpiSpinner.setMaximumSize(new Dimension(60,20));
		dpiSpinner.setPreferredSize(new Dimension(60,20));
		fontSizeBox.add(dpiSpinner);
		
		controls.add(fontSizeBox);
		JToolBar toolBar = new JToolBar(JToolBar.HORIZONTAL);
		toolBar.setFloatable(false);
		toolBar.setBorder(null);
		toolBar.add(new AbstractAction("show log") {
			private static final long serialVersionUID = -1770784057261352451L;

			public void actionPerformed(ActionEvent e) {
				logFrame.setVisible(true);
			}
		});
		toolBar.add(new AbstractAction("compile") {
			private static final long serialVersionUID = -1770784057261352451L;

			public void actionPerformed(ActionEvent e) {
				latexSourceChanged();
			}
		});
		toolBar.getComponent(0).setFocusable(false);
		controls.add(Box.createHorizontalGlue());
		controls.add(toolBar);
		add(controls, BorderLayout.NORTH);
		
		// split pane
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setDividerLocation(.5);
		splitPane.setResizeWeight(.7);
		
		// input area
		input.setBorder(new EmptyBorder(6,6,6,6));
		input.setFont(new Font("Monospaced", Font.PLAIN, input.getFont().getSize()));
		input.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ENTER && e.isShiftDown()) {
					latexSourceChanged();
				}
			}
		});
		log.setBorder(new EmptyBorder(6,6,6,6));
		splitPane.add(new JScrollPane(input), JSplitPane.TOP);
		
		// preview
		Box southBox = new Box(BoxLayout.X_AXIS);
		preview.setBorder(new EmptyBorder(20,20,20,20));
		southBox.add(Box.createHorizontalGlue());
		southBox.add(preview);
		southBox.add(Box.createHorizontalGlue());
		splitPane.add(new JScrollPane(southBox), JSplitPane.BOTTOM);
		
		add(splitPane, BorderLayout.CENTER);
		if (f!=null) setObject(f);
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
		support.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		support.removePropertyChangeListener(listener);
	}

	public void setObject(Object latexFormula) {
		latexIcon = (LatexIcon)latexFormula;
		fontSizeModel.setValue(latexIcon.getFontSize());
		input.setText(latexIcon.getLatexSource());
		log.setText(latexIcon.getLog());
		preview.setIcon(latexIcon.getIcon());
		preview.repaint();
	}
	
	private void fontSizeChanged() {
		int oldFontSize = latexIcon.getFontSize();
		int fontSize = fontSizeModel.getNumber().intValue();
		if (oldFontSize != fontSize) {
			latexIcon.setFontSize(fontSize);
			support.firePropertyChange("dpi",oldFontSize, fontSize);
			setObject(latexIcon);
		}
	}
	
	private void latexSourceChanged() {
		String oldLatexSource = latexIcon.getLatexSource();
		String newLatexSource = input.getText();
		if (!oldLatexSource.equals(newLatexSource)) {
			latexIcon.setLatexSource(newLatexSource);
			support.firePropertyChange("latexSource",oldLatexSource, newLatexSource);
			setObject(latexIcon);
		}
	}
	
	public static void main(String[] args) {
		JFrame frame = new JFrame("latex formula customizer");
		frame.getContentPane().setLayout(new BorderLayout());
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(300,300);
		LatexIconCustomizer f =
			new LatexIconCustomizer(
					new LatexIcon("$\\int_a^b f(x) dx$",300)
			);
		frame.getContentPane().add(f);
		frame.setVisible(true);
	}
}
