/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.modelling;

import java.util.HashSet;

public class Modeller2DUtility {
	
	// for SceneComponent
	static HashSet<String>expertProperties = new HashSet<String>();
	static HashSet<String>areaProperties = new HashSet<String>();
	static HashSet<String>outlineProperties = new HashSet<String>();
	static HashSet<String>pointProperties = new HashSet<String>();
	static HashSet<String>outlineAndPointProperties = new HashSet<String>();
	static HashSet<String>annotationProperties = new HashSet<String>();
	static HashSet<String>iconAndAnnotationProperties = new HashSet<String>();
	
	static HashSet<String>expertPropertiesForJPanel = new HashSet<String>();
	static HashSet<String>expertPropertiesForViewer2D = new HashSet<String>();
	
	static {
		expertProperties.add("areaDragEnabled");
		expertProperties.add("outlineDragEnabled");
		expertProperties.add("pointDragEnabled");
		expertProperties.add("annotationDragEnabled");
		expertProperties.add("iconDragEnabled");
		expertProperties.add("mouseOverEnabled");
		expertProperties.add("name");
		expertProperties.add("childCount");
		expertProperties.add("highlighted");
 
		areaProperties.add("filled");
		areaProperties.add("paint");
		areaProperties.add("highlightPaint");
		
		outlineProperties.add("outlined");
		outlineProperties.add("stroke");
		
		pointProperties.add("pointFilled");
		pointProperties.add("pointOutlined");
		pointProperties.add("pointShape");
		pointProperties.add("pointStroke");
		pointProperties.add("pointPaint");
		pointProperties.add("highlightPointPaint");
		
		outlineAndPointProperties.add("outlinePaint");
		outlineAndPointProperties.add("highlightOutlinePaint");
		
		annotationProperties.add("annotated");
		annotationProperties.add("annotationPaint");
		annotationProperties.add("highlightAnnotationPaint");
		annotationProperties.add("font");
		
		iconAndAnnotationProperties.add("annotationMargin");
		
		expertPropertiesForJPanel.add("UIClassID");
		expertPropertiesForJPanel.add("alignmentX");
		expertPropertiesForJPanel.add("alignmentY");
		expertPropertiesForJPanel.add("autoscrolls");
		expertPropertiesForJPanel.add("componentCount");
		expertPropertiesForJPanel.add("debugGraphicsOptions");
		expertPropertiesForJPanel.add("enabled");
		expertPropertiesForJPanel.add("focusable");
		expertPropertiesForJPanel.add("focusCycleRoot");
		expertPropertiesForJPanel.add("focusTraversalPolicyProvider");
		expertPropertiesForJPanel.add("focusTraversalPolicySet");
		expertPropertiesForJPanel.add("font");
		expertPropertiesForJPanel.add("foreground");
		expertPropertiesForJPanel.add("height");
		expertPropertiesForJPanel.add("inheritsPopupMenu");
		expertPropertiesForJPanel.add("managingFocus");
		expertPropertiesForJPanel.add("name");
		expertPropertiesForJPanel.add("opaque");
		expertPropertiesForJPanel.add("optimizedDrawingEnabled");
		expertPropertiesForJPanel.add("paintingTile");
		expertPropertiesForJPanel.add("requestFocusEnabled");
		expertPropertiesForJPanel.add("toolTipText");
		expertPropertiesForJPanel.add("validateRoot");
		expertPropertiesForJPanel.add("doubleBuffered");
		expertPropertiesForJPanel.add("verifyInputWhenFocusTarget");
		expertPropertiesForJPanel.add("visible");
		expertPropertiesForJPanel.add("width");
		expertPropertiesForJPanel.add("x");
		expertPropertiesForJPanel.add("y");

		expertPropertiesForViewer2D.addAll(expertPropertiesForJPanel);
		expertPropertiesForViewer2D.add("keepingAspectRatio");
		expertPropertiesForViewer2D.add("translateToolEnabled");
		expertPropertiesForViewer2D.add("scaleToolEnabled");
		expertPropertiesForViewer2D.add("menuToolEnabled");
	}
	
	public static HashSet<String> getExpertProperties() {
		return expertProperties;
	}
	
	public static HashSet<String> getAreaProperties() {
		return areaProperties;
	}
	
	public static HashSet<String> getOutlineProperties() {
		return outlineProperties;
	}
	
	public static HashSet<String> getPointProperties() {
		return pointProperties;
	}

	public static HashSet<String> getOutlineAndPointProperties() {
		return outlineAndPointProperties;
	}

	public static HashSet<String> getIconAndAnnotationProperties() {
		return iconAndAnnotationProperties;
	}

	public static HashSet<String> getAnnotationProperties() {
		return annotationProperties;
	}

	public static HashSet<String> getExpertPropertiesForJPanel() {
		return expertPropertiesForJPanel;
	}

	public static HashSet<String> getExpertPropertiesForViewer2D() {
		return expertPropertiesForViewer2D;
	}
}
