/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.numerical;



import java.awt.geom.Rectangle2D;
import java.util.Collections;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.jtem.beans.InspectorPanel;
import de.jtem.java2d.ViewportChangeEvent;
import de.jtem.java2d.ViewportChangeListener;
import de.jtem.java2dx.beans.Viewer2DWithInspector;
public class Viewer2DWithContourPlot extends Viewer2DWithInspector {

	private static final long serialVersionUID = -2758190599522848662L;
	final static Set excludedPropertiesForContourPlot = Collections.singleton("name");
	private ContourPlot plot = new ContourPlot();
	private boolean contourPlotEnabled = false;
	private ViewportChangeListener plotListener = new ViewportChangeListener() {

		public void viewportChange(ViewportChangeEvent e) {
			plot.setRectangle(getViewport());
		}
	};
	
	public Viewer2DWithContourPlot() {
		this(null);
	}
	
	public Viewer2DWithContourPlot(Rectangle2D.Double r) {
		this(r,null);
	}

	public Viewer2DWithContourPlot(Rectangle2D.Double r, InspectorPanel i) {
		super(r,i);
		
		setContourPlotEnabled(true);
		addInspectActionToMenu(plot,"level curves",excludedPropertiesForContourPlot);
		getInspectorPanel().addChangeListener(
    			new ChangeListener() {
    				public void stateChanged(ChangeEvent event) {
    					if (event.getSource() instanceof ContourPlot) {
    						repaint();
    					}
    				}
    			}
    	);
	}
	
	public ContourPlot getContourPlot() {
		return plot;
	}
	
	public static void main(String[] args) {
		Viewer2DWithContourPlot viewer = new Viewer2DWithContourPlot(
				new Rectangle2D.Double(-5,-5,10,10)
		);
		
		RealFunctionOnRectangle f = new RealFunctionOnRectangle() {
			
			public double value(double x, double y) {
				return Math.cos(x)+Math.sin(y);
			}
			
		};
		
		viewer.getContourPlot().setFunction(f);
		JFrame frame = new JFrame("contour plot");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(600,600);
		frame.getContentPane().add(viewer);
		frame.setVisible(true);
	}

	public boolean isContourPlotEnabled() {
		return contourPlotEnabled;
	}

	public void setContourPlotEnabled(boolean b) {
		if (b && !contourPlotEnabled) {
			addViewportChangeListener(plotListener);
		    getBackdrop().addChild(plot.getViewScene());
			contourPlotEnabled = true;
			getContourPlot().setRectangle(getViewport());
			getContourPlot().functionChanged();
			
			repaint();
		}
		if (!b && contourPlotEnabled) {
			removeViewportChangeListener(plotListener);
			getBackdrop().removeChild(plot.getViewScene());
			contourPlotEnabled = false;
			getContourPlot().functionChanged();
			repaint();
		}
	}
}
