/**
This file is part of a jTEM project.
All jTEM projects are licensed under the FreeBSD license 
or 2-clause BSD license (see http://www.opensource.org/licenses/bsd-license.php). 

Copyright (c) 2006-2010, Technische Universität Berlin, jTEM
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

-	Redistributions of source code must retain the above copyright notice, 
	this list of conditions and the following disclaimer.

-	Redistributions in binary form must reproduce the above copyright notice, 
	this list of conditions and the following disclaimer in the documentation 
	and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
**/

package de.jtem.java2dx.numerical;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;

import javax.swing.JFrame;
import javax.swing.JPopupMenu;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.jtem.beans.InspectorPanel;
import de.jtem.java2d.SceneComponent;
import de.jtem.java2dx.ContextMenu;
import de.jtem.java2dx.beans.Viewer2DWithInspector;

public class Viewer2DWithFunctionPlotter extends Viewer2DWithInspector {

	private static final long serialVersionUID = -2758190599522848662L;
	private HashSet functions = new HashSet();
	
	public Viewer2DWithFunctionPlotter(Rectangle2D.Double r, InspectorPanel i) {
		super(r, i);
		setEncompassMarginX(0);
		setKeepingAspectRatio(false);
		getInspectorPanel().addChangeListener(
    			new ChangeListener() {
    				public void stateChanged(ChangeEvent event) {
    					if (event.getSource() instanceof FunctionPlot) {
    						repaint();
    					}
    				}
    			}
    	);
	}

	public Viewer2DWithFunctionPlotter(Rectangle2D.Double r) {
		this(r,null);
	}

	public Viewer2DWithFunctionPlotter() {
		this(null);
	}
	
	public void addFunctionPlot(final FunctionPlot f) {
		SceneComponent view = f.getViewScene();
		view.setOutlineDragEnabled(true);
		JPopupMenu m = new JPopupMenu();
		m.add(getInspectAction(f,"inspect"));
		view.addDragListener(new ContextMenu(m));
		addViewportChangeListener(f);
		functions.add(f.getModel());
		getRoot().addChild(view);
	}
	
	public void removeFunctionPlot(FunctionPlot f) {
		removeViewportChangeListener(f);
		functions.remove(f.getModel());
		getBackdrop().removeChild(f.getViewScene());
	}
	
	public static void main(String[] args) {
		Viewer2DWithFunctionPlotter viewer = new Viewer2DWithFunctionPlotter();
		viewer.setName("viewer");
		RealFunctionOnInterval f = new RealFunctionOnInterval() {
			public double value(double x) {
				return Math.sin(x);
			}
		};
		FunctionPlot plot = new FunctionPlot(f,new Rectangle2D.Double(-10,-2,20,4));
		plot.setPaint(Color.red);
		plot.setName("sin");
		plot.setStroke(new BasicStroke(2));
		viewer.addFunctionPlot(plot);
		JFrame frame = new JFrame("function plotter");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(600,600);
		frame.getContentPane().add(viewer);
		frame.setVisible(true);
	}
}